<?php
/**
 * $Horde$
 *
 * Add Link API for Horde_Links into Klutz
 *
 * Copyright 2003, Jeroen Huinink <j.huinink@wanadoo.nl>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 * 
 * @author  Jeroen Huinink <j.huinink@wanadoo.nl>
 * @version $Revision$
 * @since   Horde 3.0
 * @package horde.links
 */

/* TODO: This is a quick and dirty implementation. We drag a lot of information
 * in the url. We might want to consider moving this into _SESSION variables.
 * The layout of the page is also rather basic.
 */
define('KLUTZ_BASE', dirname(__FILE__));
require_once KLUTZ_BASE . '/lib/base.php';
require_once KLUTZ_BASE . '/config/html.php';

$actionID = Horde::getFormData('actionID');

$link_type = Horde::getFormData('link_type');
$from_application = Horde::getFormData('from_application');
$from_parameters = unserialize(Horde::getFormData('from_parameters'));
$return_url = Horde::getFormData('url');

$to_application = Horde::getFormData('to_application');
if (!empty($to_application)) {
    require_once HORDE_BASE . '/lib/Links.php';
    $links = &Horde_Links::singleton($registry->getApp());
    $to_parameters = unserialize(Horde::getFormData('to_parameters'));
    $links->addLink($from_application, $from_parameters, $to_application, $to_parameters, $link_type);
?>
<h1>Add Link</h1>
<?php
if ($registry->hasMethod($to_application . '/getLinkDescription')) {
?>
<p>Added a "<?php echo $link_type ?>" link to <?php echo $registry->call($to_application. '/getLinkDescription', $to_parameters); ?>.</p>
<?php } ?>
<p><a href="<?php echo $return_url ?>" class="button"><?php echo _("Continue") ?></a></p>
<?php
} else {

    // Page variables.
    $title = _("Today's Comics");
    $js_onLoad = null;
    
    $date = Horde::getFormData('date');
    $index = Horde::getFormData('index');
    $date_format = 'D, d M Y H:i:s T';
    $daystokeep = 365;
    if (!empty($conf['backend']['daystokeep'])) {
        $daystokeep = $conf['backend']['daystokeep'];
    }
    
    if (is_null($actionID)) {
        if (is_null($date) && is_null($index)) {
            $actionID = KLUTZ_DISPLAY_MAIN;
        } elseif (empty($date) && !empty($index)) {
            $actionID = KLUTZ_DISPLAY_COMIC;
        } elseif (empty($index) && !empty($date)) {
            $actionID = KLUTZ_DISPLAY_DAY;
        } else {
            $actionID = KLUTZ_DISPLAY_IMAGE;
        }
    }
    
    // Make sure we have a date set.
    if (empty($date)) { $date = mktime(0, 0, 0); }
    if (!is_numeric($date)) { $date = strtotime($date); }
    
    if ($actionID != KLUTZ_DISPLAY_IMAGE) {
        require KLUTZ_TEMPLATES . '/common-header.inc';
        require_once HORDE_BASE . '/lib/Menu.php';
        require KLUTZ_TEMPLATES . '/addlink/menu.inc';
        $notification->notify();
    
        // Figure out timestamps for navigation links.
        $time = time();
        $dates = $storage->listDates($time, $time - (86400*$daystokeep));
        $first_day = $dates[0];
        $last_day = $dates[count($dates) - 1];
    
        $d = getdate($date);
        $prev_month = mktime(0, 0, 0, $d['mon'] - 1, 1, $d['year']);
        $next_month = mktime(0, 0, 0, $d['mon'] + 1, 1, $d['year']);
        $yesterday  = $date - 86400;
        $tomorrow = $date + 86400;
    
    
        $url = Horde::selfUrl(false);
        $url = Horde::addParameter($url, "actionID=$actionID");
        $url = Horde::addParameter($url, 'url=' . $return_url);
        $url = Horde::addParameter($url, 'link_type=' . $link_type);
        $url = Horde::addParameter($url, 'from_application=' . $from_application);
        $url = Horde::addParameter($url, 'from_parameters=' . serialize($from_parameters)); 
        $url = Horde::addParameter($url, 'to_application=' . $registry->getParam('provides'));

    
        $first_day_url = null;
        $prev_month_url = null;
        $yesterday_url = null;
        $tomorrow_url = null;
        $next_month_url = null;
        $last_day_url = null;
    
    }
    
    switch ($actionID) {
    case KLUTZ_DISPLAY_IMAGE:
        $index = Horde::getFormData('index');
        $image = $storage->retrieveImage($index, $date);
    
        if (is_object($image)) {
            header("Content-type: " . $image->type);
            if ($image->lastmodified > 0) {
                header('Last-Modified: ' .
                    gmdate($date_format, $image->lastmodified));
            }
            header('Expires: ' . gmdate($date_format, time() + 172800));
            header('Cache-Control: public, max-age=172800');
            header('Pragma:');
            print $image->data;
        } elseif (is_string($image) && substr($image, 0, 4) == 'http') {
            header("Location: $image");
            exit();
        } else {
            // Do some kind of error handling here.
        }
        break;
    
    case KLUTZ_DISPLAY_MAIN:
        if ($prefs->getValue('show_unselected')) {
            $comics = $klutz->listEnabled();
        } else {
            // Get the list of comics to display.
            $comics = explode("\t", $prefs->getValue('viewcomics'));
            if (count($comics) == 1 && empty($comics[0])) {
                $comics = null;
            }
            $comics = $klutz->listEnabled($comics);
        }
    
        $selected = explode("\t", $prefs->getValue('viewcomics'));
        if (count($selected) == 1 && empty($selected[0])) {
            $selected = $comics;
        }
    
        $visited = explode("\t", $prefs->getValue('datesvisited'));
        if (count($visited) == 1 && empty($visited[0])) {
            $visited = array();
        }
    
        $comics = $klutz->getProperty($comics, array('name', 'author'));
    
        // Date navigation.
        if (count($storage->listDates($prev_month)) > 0) {
            $prev_month_url = Horde::addParameter($url, "date=$prev_month");
        }
        if (count($storage->listDates($next_month)) > 0) {
            $next_month_url = Horde::addParameter($url, "date=$next_month");
        }
    
        require KLUTZ_TEMPLATES . '/addlink/main.inc';
        break;
    
    case KLUTZ_DISPLAY_DAY:
        // Display the navbar.
        if (in_array($yesterday, $storage->listDates($yesterday))) {
            $yesterday_url = Horde::addParameter($url, "date=$yesterday");
        }
        if (in_array($tomorrow, $storage->listDates($tomorrow))) {
            $tomorrow_url = Horde::addParameter($url, "date=$tomorrow");
        }
        if (date('dmY', $date) != date('dmY', $first_day)) {
            $first_day_url = Horde::addParameter($url, "date=$first_day");
        }
        if (date('dmY', $date) != date('dmY', $last_day)) {
            $last_day_url = Horde::addParameter($url, "date=$last_day");
        }
    
        require KLUTZ_TEMPLATES . '/addlink/nav_bar.inc';
    
        // Used for tracking dates we've already looked at.
        $today = date('Ymd', $date);
    
        // Get the list of comics to display.
        $comics = explode("\t", $prefs->getValue('viewcomics'));
        if (count($comics) == 1 && empty($comics[0])) {
            $comics = null;
        }
    
        // Update prefs to mark this date as visited.
        $visited = explode("\t", $prefs->getValue('datesvisited'));
        if (count($visited) == 1 && empty($visited[0])) {
            $visited = array();
        }
        if (!$prefs->isLocked('datesvisited') && !in_array($today, $visited)) {
            array_push($visited, $today);
            $prefs->setValue('datesvisited', implode("\t", $visited));
        }
    
        foreach ($klutz->listEnabled($comics, $date) as $index) {
            $name = $klutz->getProperty($index, 'name');
            $author = $klutz->getProperty($index, 'author');
            $homepage = $klutz->getProperty($index, 'homepage');
            if ($storage->imageExists($index, $date)) {
                $size = $storage->imageSize($index, $date);
                $url = Horde::applicationURL('comics.php');
                $url = Horde::addParameter($url, 'actionID=' . KLUTZ_DISPLAY_IMAGE);
                $url = Horde::addParameter($url, "date=$date");
                $url = Horde::addParameter($url, "index=$index");
                require KLUTZ_TEMPLATES . '/addlink/comic.inc';
            } elseif ($klutz->getProperty($index, 'days') == 'random') {
                @include KLUTZ_TEMPLATES . '/addlink/random.inc';
            } else {
                require KLUTZ_TEMPLATES . '/addlink/missing.inc';
            }
        }
        require KLUTZ_TEMPLATES . '/addlink/nav_bar.inc';
        break;
    
    case KLUTZ_DISPLAY_COMIC:
        // Get a list of the available comics the user reads
        $comics = explode("\t", $prefs->getValue('viewcomics'));
        if ((count($comics) == 1 && empty($comics[0]))
            || !in_array($index, $comics)) {
            $comics = null;
        }
        $comics = $klutz->listEnabled($comics);
        $i = array_search($index, $comics);
        if ($i !== false) {
            $first_comic = null;
            $prev_comic = null;
            $next_comic = null;
            $last_comic = null;
    
            $curl = Horde::addParameter($url, "date=$date");
            if ($comics[0] != $index) {
                $first_comic = Horde::addParameter($curl, 'index=' . $comics[0]);
            }
            if ($i > 0 && $comics[$i-1] != $index) {
                $prev_comic = Horde::addParameter($curl, 'index=' . $comics[$i-1]);
            }
            if ($i < (count($comics) - 1) && $comics[$i+1] != $index) {
                $next_comic = Horde::addParameter($curl, 'index=' . $comics[$i+1]);
            }
            $l = count($comics) - 1;
            if ($comics[$l] != $index) {
                $last_comic = Horde::addParameter($curl, 'index=' . $comics[$l]);
            }
        } else {
            $notification->push(_("This comic doesn't exist or is disabled."),
                                'horde.error');
            $url = Horde::selfUrl(false);
            $url = Horde::addParameter($url, "date=$date");
            $url = Horde::addParameter($url, "actionID=" . KLUTZ_DISPLAY_MAIN);
            $url = Horde::addParameter($url, 'url=' . $return_url);
            $url = Horde::addParameter($url, 'link_type=' . $link_type);
            $url = Horde::addParameter($url, 'from_application=' . $from_application);
            $url = Horde::addParameter($url, 'from_parameters=' . serialize($from_parameters)); 
            $url = Horde::addParameter($url, 'to_application=' . $registry->getParam('provides'));
            header("Location: $url");
        }
    
        $comic_select = null;
        if (count($comics) > 4) {
            $comic_select = '<select name="index" onchange="this.form.submit()">';
            foreach ($comics as $c) {
                $comic_select .= ' <option value="' . $c . '"';
                if ($c == $index) {
                    $comic_select .= ' selected="selected"';
                }
                $comic_select .= '>' . $klutz->getProperty($c, 'name');
                $comic_select .= '</option>\n';
            }
            $comic_select .= '</select>';
        }
    
        // Display the navbar.
        $url = Horde::addParameter($url, "index=$index");
        if (date('dmY', $date) != date('dmY', $first_day)) {
            $first_day_url = Horde::addParameter($url, "date=$first_day");
        }
        if (date('dmY', $date) != date('dmY', $last_day)) {
            $last_day_url = Horde::addParameter($url, "date=$last_day");
        }
        if (count($storage->listDates($prev_month)) > 0) {
            $prev_month_url = Horde::addParameter($url, "date=$prev_month");
        }
        if (count($storage->listDates($next_month)) > 0) {
            $next_month_url = Horde::addParameter($url, "date=$next_month");
        }
        require KLUTZ_TEMPLATES . '/addlink/nav_bar.inc';
    
        $name = $klutz->getProperty($index, 'name');
        $author = $klutz->getProperty($index, 'author');
        $homepage = $klutz->getProperty($index, 'homepage');
        foreach ($storage->listDates($date) as $date) {
            if ($storage->imageExists($index, $date)) {
                $size = $storage->imageSize($index, $date);
                $url = Horde::applicationURL('comics.php');
                $url = Horde::addParameter($url, 'actionID=' . KLUTZ_DISPLAY_IMAGE);
                $url = Horde::addParameter($url, "date=$date");
                $url = Horde::addParameter($url, "index=$index");
                require KLUTZ_TEMPLATES . '/addlink/comic.inc';
            } elseif ($klutz->getProperty($index, 'days') == 'random') {
                @include KLUTZ_TEMPLATES . '/addlink/random.inc';
            } elseif (!in_array(String::lower(date('D', $date)), $klutz->getProperty($index, 'days'))) {
                continue;
            } else {
                include KLUTZ_TEMPLATES . '/addlink/missing.inc';
            }
        }
        require KLUTZ_TEMPLATES . '/addlink/nav_bar.inc';
        break;
    }
    
    if ($actionID != KLUTZ_DISPLAY_IMAGE) {
        require KLUTZ_TEMPLATES . '/common-footer.inc';
    }
}
